@@ -3,7 +3,6 @@ try:
 except ImportError:
     import json
 import re
-
 from redis import Redis
 
 from redis_completion.stop_words import STOP_WORDS as _STOP_WORDS
@@ -26,32 +25,26 @@ class RedisEngine(object):
     http://patshaughnessy.net/2011/11/29/two-ways-of-using-redis-to-build-a-nosql-autocomplete-search-index
     """
     def __init__(self, min_length=2, prefix='ac', stop_words=None, cache_timeout=300, **conn_kwargs):
-        self.conn_kwargs = conn_kwargs
-        self.client = self.get_client()
-
         self.min_length = min_length
         self.prefix = prefix
         self.stop_words = (stop_words is None) and DEFAULT_STOP_WORDS or stop_words
+
+        self.conn_kwargs = conn_kwargs
+        self.client = self.get_client()
+
         self.cache_timeout = cache_timeout
 
         self.data_key = '%s:d' % self.prefix
         self.title_key = '%s:t' % self.prefix
         self.search_key = lambda k: '%s:s:%s' % (self.prefix, k)
+        self.cache_key = lambda pk, bk: '%s:c:%s:%s' % (self.prefix, pk, bk)
+
+        self.kcombine = lambda _id, _type: ''.join([str(_id), '\x01', str(_type)])
+        self.ksplit = lambda k: k.split('\x01', 1)
 
     def get_client(self):
         return Redis(**self.conn_kwargs)
 
-    def flush(self, everything=False, batch_size=1000):
-        if everything:
-            return self.client.flushdb()
-
-        # this could be expensive :-(
-        keys = self.client.keys('%s:*' % self.prefix)
-
-        # batch keys
-        for i in range(0, len(keys), batch_size):
-            self.client.delete(*keys[i:i+batch_size])
-
     def score_key(self, k, max_size=20):
         k_len = len(k)
         a = ord('a') - 2
@@ -80,7 +73,18 @@ class RedisEngine(object):
             yield w[:i+ml]
         yield w
 
-    def store(self, obj_id, title=None, data=None):
+    def flush(self, everything=False, batch_size=1000):
+        if everything:
+            return self.client.flushdb()
+
+        # this could be expensive :-(
+        keys = self.client.keys('%s:*' % self.prefix)
+
+        # batch keys
+        for i in range(0, len(keys), batch_size):
+            self.client.delete(*keys[i:i+batch_size])
+
+    def store(self, obj_id, title=None, data=None, obj_type=None):
         pipe = self.client.pipeline()
 
         if title is None:
@@ -90,6 +94,8 @@ class RedisEngine(object):
 
         title_score = self.score_key(self.create_key(title))
 
+        obj_id = self.kcombine(obj_id, obj_type or '')
+
         pipe.hset(self.data_key, obj_id, data)
         pipe.hset(self.title_key, obj_id, title)
 
@@ -99,11 +105,11 @@ class RedisEngine(object):
 
         pipe.execute()
 
-    def store_json(self, obj_id, title, data_dict):
-        return self.store(obj_id, title, json.dumps(data_dict))
+    def store_json(self, obj_id, title, data_dict, obj_type=None):
+        return self.store(obj_id, title, json.dumps(data_dict), obj_type)
 
-    def remove(self, obj_id):
-        obj_id = str(obj_id)
+    def remove(self, obj_id, obj_type=None):
+        obj_id = self.kcombine(obj_id, obj_type or '')
         title = self.client.hget(self.title_key, obj_id) or ''
         keys = []
 
@@ -115,29 +121,51 @@ class RedisEngine(object):
                 else:
                     self.client.zrem(key, obj_id)
 
-        # finally, remove the data from the data key
         self.client.hdel(self.data_key, obj_id)
         self.client.hdel(self.title_key, obj_id)
 
-    def search(self, phrase, limit=None, filters=None, mappers=None):
-        """
-        Wrap our search & results with prefixing
-        """
+    def exists(self, obj_id, obj_type=None):
+        obj_id = self.kcombine(obj_id, obj_type or '')
+        return self.client.hexists(self.data_key, obj_id)
+
+    def get_cache_key(self, phrases, boosts):
+        if boosts:
+            boost_key = '|'.join('%s:%s' % (k, v) for k, v in sorted(boosts.items()))
+        else:
+            boost_key = ''
+        phrase_key = '|'.join(phrases)
+        return self.cache_key(phrase_key, boost_key)
+
+    def search(self, phrase, limit=None, filters=None, mappers=None, boosts=None):
         cleaned = self.clean_phrase(phrase)
         if not cleaned:
             return []
 
-        new_key = self.search_key('|'.join(cleaned))
-        if not self.client.exists(new_key):
-            self.client.zinterstore(new_key, map(self.search_key, cleaned))
-            self.client.expire(new_key, self.cache_timeout)
+        if len(cleaned) == 1 and not boosts:
+            new_key = self.search_key(cleaned[0])
+        else:
+            new_key = self.get_cache_key(cleaned, boosts)
+            if not self.client.exists(new_key):
+                # zinterstore also takes {k1: wt1, k2: wt2}
+                self.client.zinterstore(new_key, map(self.search_key, cleaned))
+                self.client.expire(new_key, self.cache_timeout)
+
+        if boosts:
+            pipe = self.client.pipeline()
+            for raw_id, score in self.client.zrange(new_key, 0, -1, withscores=True):
+                orig_score = score
+                for part in self.ksplit(raw_id):
+                    if part and part in boosts:
+                        score *= 1 / boosts[part]
+                if orig_score != score:
+                    pipe.zadd(new_key, raw_id, score)
+            pipe.execute()
 
         ct = 0
         data = []
 
-        # grab the data for each object
-        for obj_id in self.client.zrange(new_key, 0, -1):
-            raw_data = self.client.hget(self.data_key, obj_id)
+        for raw_id in self.client.zrange(new_key, 0, -1):
+            raw_data = self.client.hget(self.data_key, raw_id)
             if not raw_data:
                 continue
 
@@ -162,8 +190,8 @@ class RedisEngine(object):
 
         return data
 
-    def search_json(self, phrase, limit=None, filters=None, mappers=None):
+    def search_json(self, phrase, limit=None, filters=None, mappers=None, boosts=None):
         if not mappers:
             mappers = []
         mappers.insert(0, json.loads)
-        return self.search(phrase, limit, filters, mappers)
+        return self.search(phrase, limit, filters, mappers, boosts)
