@@ -8,9 +8,12 @@ stop_words = set(['a', 'an', 'the', 'of'])
 
 class RedisCompletionTestCase(TestCase):
     def setUp(self):
-        self.engine = RedisEngine(prefix='testac', db=15)
+        self.engine = self.get_engine()
         self.engine.flush()
 
+    def get_engine(self):
+        return RedisEngine(prefix='testac', db=15)
+
     def store_data(self, id=None):
         test_data = (
             (1, 'testing python'),
@@ -58,6 +61,48 @@ class RedisCompletionTestCase(TestCase):
         results = self.engine.search_json('missing')
         self.assertEqual(results, [])
 
+    def test_boosting(self):
+        test_data = (
+            (1, 'test alpha', 't1'),
+            (2, 'test beta', 't1'),
+            (3, 'test gamma', 't1'),
+            (4, 'test delta', 't1'),
+            (5, 'test alpha', 't2'),
+            (6, 'test beta', 't2'),
+            (7, 'test gamma', 't2'),
+            (8, 'test delta', 't2'),
+            (9, 'test alpha', 't3'),
+            (10, 'test beta', 't3'),
+            (11, 'test gamma', 't3'),
+            (12, 'test delta', 't3'),
+        )
+        for obj_id, title, obj_type in test_data:
+            self.engine.store_json(obj_id, title, {
+                'obj_id': obj_id,
+                'title': title,
+            }, obj_type)
+
+        def assertExpected(results, id_list):
+            self.assertEqual([r['obj_id'] for r in results], id_list)
+
+        results = self.engine.search_json('alp')
+        assertExpected(results, [1, 5, 9])
+
+        results = self.engine.search_json('alp', boosts={'t2': 1.1})
+        assertExpected(results, [5, 1, 9])
+
+        results = self.engine.search_json('test', boosts={'t3': 1.5, 't2': 1.1})
+        assertExpected(results, [9, 10, 12, 11, 5, 6, 8, 7, 1, 2, 4, 3])
+
+        results = self.engine.search_json('alp', boosts={'t1': 0.5})
+        assertExpected(results, [5, 9, 1])
+
+        results = self.engine.search_json('alp', boosts={'t1': 1.5, 't3': 1.6})
+        assertExpected(results, [9, 1, 5])
+
+        results = self.engine.search_json('alp', boosts={'t3': 1.5, '5': 1.6})
+        assertExpected(results, [5, 9, 1])
+
     def test_limit(self):
         self.store_data()
 
@@ -128,6 +173,21 @@ class RedisCompletionTestCase(TestCase):
             {'obj_id': 3, 'title': 'web testing python code', 'secret': 'herp'},
         ])
 
+    def test_clean_phrase(self):
+        self.assertEqual(self.engine.clean_phrase('abc def ghi'), ['abc', 'def', 'ghi'])
+
+        self.assertEqual(self.engine.clean_phrase('a A tHe an a'), [])
+        self.assertEqual(self.engine.clean_phrase(''), [])
+
+        self.assertEqual(
+            self.engine.clean_phrase('The Best of times, the blurst of times'),
+            ['best', 'times', 'blurst', 'times'])
+
+    def test_exists(self):
+        self.assertFalse(self.engine.exists('test'))
+        self.engine.store('test')
+        self.assertTrue(self.engine.exists('test'))
+
     def test_removing_objects_in_depth(self):
         # want to ensure that redis is cleaned up and does not become polluted
         # with spurious keys when objects are removed
@@ -153,13 +213,3 @@ class RedisCompletionTestCase(TestCase):
 
         self.engine.remove(1)
         self.assertEqual(len(redis_client.keys()), initial_key_count)
-
-    def test_clean_phrase(self):
-        self.assertEqual(self.engine.clean_phrase('abc def ghi'), ['abc', 'def', 'ghi'])
-
-        self.assertEqual(self.engine.clean_phrase('a A tHe an a'), [])
-        self.assertEqual(self.engine.clean_phrase(''), [])
-
-        self.assertEqual(
-            self.engine.clean_phrase('The Best of times, the blurst of times'),
-            ['best', 'times', 'blurst', 'times'])
